@@ -5,7 +5,6 @@ import gzip
 import itertools
 import mmap
 import os
-import platform
 import signal
 import sys
 import tempfile
@@ -25,17 +24,16 @@ except ImportError:
     io = None
 
 try:
-    try:
-        from cStringIO import StringIO
-    except ImportError:
-        from StringIO import StringIO
+    from StringIO import StringIO
 except ImportError:
-    from io import StringIO
+    class StringIO(object):
+        pass
+
 
 try:
     from io import BytesIO
 except ImportError:
-    BytesIO = StringIO
+    from cStringIO import StringIO as BytesIO
 
 
 import numpy as np
@@ -471,7 +469,7 @@ def fileobj_closed(f):
     elif hasattr(f, 'fp') and hasattr(f.fp, 'closed'):
         return f.fp.closed
     else:
-        return True
+        return False
 
 
 def fileobj_mode(f):
@@ -754,7 +752,14 @@ def _array_to_file(arr, outfile):
         # treat as file-like object with "write" method and write the array
         # via its buffer interface
         def write(a, f):
-            f.write(a.flatten().view(np.ubyte))
+            # StringIO in Python 2.5 asks 'if not s' which fails for a Numpy
+            # array; test ahead of time if the array is empty, and pass in the
+            # array buffer directly
+            if isinstance(f, StringIO):
+                if len(a):
+                    f.write(a.data)
+            else:
+                f.write(a)
 
     # Implements a workaround for a bug deep in OSX's stdlib file writing
     # functions; on 64-bit OSX it is not possible to correctly write a number
@@ -764,8 +769,8 @@ def _array_to_file(arr, outfile):
     # implemented there yet: https://github.com/astropy/astropy/issues/839
     osx_write_limit = (2 ** 32) - 1
 
-    if (sys.platform == 'darwin' and platform.architecture()[0] == '64bit' and
-            arr.nbytes >= osx_write_limit + 1 and arr.nbytes % 4096 == 0):
+    if (sys.platform == 'darwin' and arr.nbytes >= osx_write_limit + 1 and
+            arr.nbytes % 4096 == 0):
         idx = 0
         # chunksize is a count of elements in the array, not bytes
         chunksize = osx_write_limit // arr.itemsize
@@ -790,6 +795,9 @@ def _write_string(f, s):
         s = encode_ascii(s)
     elif not binmode and not isinstance(f, unicode):
         s = decode_ascii(s)
+    elif isinstance(f, StringIO) and isinstance(s, np.ndarray):
+        # Workaround for StringIO/ndarray incompatibility
+        s = s.data
     f.write(s)
 
 
@@ -1006,3 +1014,27 @@ if sys.version_info[:2] < (2, 6):
 
             return cls_ns[property_name]
     __builtin__.property = property
+
+
+    # Provide an implementation of izip_longest
+    class ZipExhausted(Exception):
+        pass
+
+    def izip_longest(*args, **kwds):
+        # izip_longest('ABCD', 'xy', fillvalue='-') --> Ax By C- D-
+        fillvalue = kwds.get('fillvalue')
+        counter = [len(args) - 1]
+        def sentinel():
+            if not counter[0]:
+                raise ZipExhausted
+            counter[0] -= 1
+            yield fillvalue
+        fillers = itertools.repeat(fillvalue)
+        iterators = [itertools.chain(it, sentinel(), fillers) for it in args]
+        try:
+            while iterators:
+                yield tuple(map(lambda it: it.next(), iterators))
+        except ZipExhausted:
+            pass
+
+    itertools.izip_longest = izip_longest
